/*
** Graphical User Interface using MUI
** for Printer ACEpansion
*/

#include <clib/alib_protos.h>

#include <proto/exec.h>
#include <proto/utility.h>
#include <proto/intuition.h>
#include <proto/muimaster.h>

#include <libraries/asl.h>
#include <acepansion/lib_header.h>

#include "acepansion.h"
#include "interface.h"
#define CATCOMP_NUMBERS
#include "generated/locale_strings.h" /* catalog string ids */



struct Library *IntuitionBase;
struct Library *UtilityBase;
struct Library *MUIMasterBase;



struct MUI_CustomClass *MCC_PrinterClass = NULL;



/*
** MCC_PrinterClass definitions
*/
#define PrinterObject NewObject(MCC_PrinterClass->mcc_Class, NULL

#define MUIA_Printer_Plugin (TAG_USER | 0x0000) // [I..] struct ACEpansionPlugin *, Reference to the plugin managing the GUI (mandatory)

#define MUIM_Printer_Notify (TAG_USER | 0x0100) // struct MUIP_Printer_Notify, notify the plugin or the GUI about changes
    #define MUIV_Printer_Notify_ToGUI    0 // Notify the GUI
    #define MUIV_Printer_Notify_ToPlugin 1 // Notify the plugin
        // For both directions
        #define MUIV_Printer_Notify_FileName 10 // Notify about printer file name

struct MUIP_Printer_Notify
{
    IPTR id;
    IPTR direction;
    IPTR type;
    CONST_STRPTR fileName;
};

struct PrinterData
{
    struct ACEpansionPlugin *plugin;

    Object *STR_PrinterFile;
};



/*
** MCC_PrinterClass methods
*/
static IPTR mNotify(struct IClass *cl, Object *obj, struct MUIP_Printer_Notify *msg)
{
    struct PrinterData *data = INST_DATA(cl,obj);

    switch(msg->direction)
    {
        case MUIV_Printer_Notify_ToGUI:
            switch(msg->type)
            {
                case MUIV_Printer_Notify_FileName:
                    nnset(data->STR_PrinterFile, MUIA_String_Contents, msg->fileName);
                    break;
            }
            break;
        case MUIV_Printer_Notify_ToPlugin:
            switch(msg->type)
            {
                case MUIV_Printer_Notify_FileName:
                    Plugin_SetPrinterFileName(data->plugin, msg->fileName);
                    break;
            }
            break;
    }

    return 0;
}

static IPTR mNew(struct IClass *cl, Object *obj, Msg msg)
{

    struct TagItem *tags,*tag;
    struct PrinterData *data;

    struct ACEpansionPlugin *plugin = NULL;

    Object *STR_PrinterFile;

    // Parse initial taglist
    for(tags=((struct opSet *)msg)->ops_AttrList; (tag=NextTagItem(&tags)); )
    {
        switch(tag->ti_Tag)
        {
            case MUIA_Printer_Plugin:
                plugin = (struct ACEpansionPlugin *)tag->ti_Data;
                break;                              
        }
    }

    if(!plugin) return 0;

    obj = (Object *)DoSuperNew(cl,obj,
        MUIA_Group_Columns, 2,
        GroupFrameT(GetString(MSG_GROUP_TITLE)),
        Child, Label2(GetString(MSG_LABEL_FILE)),
        Child, PopaslObject,
            MUIA_Popstring_String, STR_PrinterFile = StringObject,
                StringFrame,
                MUIA_CycleChain, TRUE,
                End,
            MUIA_Popstring_Button, TextObject,
                ButtonFrame,
                MUIA_Weight, 0,
                MUIA_InputMode , MUIV_InputMode_RelVerify,
                MUIA_Background, MUII_ButtonBack,
                MUIA_CycleChain, TRUE,
                MUIA_Text_Contents, "\33I[6:19]", // MUII_PopFile
                TAG_END),
            ASLFR_TitleText, GetString(MSG_ASL_REQUEST),
            ASLFR_DoSaveMode, TRUE,
            MUIA_ShortHelp, GetString(MSG_FILE_HELP),
            End,
        TAG_MORE,((struct opSet *)msg)->ops_AttrList);

    if(!obj) return 0;

    data = INST_DATA(cl,obj);

    data->plugin = plugin;

    data->STR_PrinterFile = STR_PrinterFile;

    DoMethod(STR_PrinterFile, MUIM_Notify, MUIA_String_Acknowledge, MUIV_EveryTime,
             obj, 4, MUIM_Printer_Notify, MUIV_Printer_Notify_ToPlugin,
                     MUIV_Printer_Notify_FileName, MUIV_TriggerValue);

    return (IPTR)obj;
}



/*
** MUI Custom Class dispatcher, creation & destruction
*/
DISPATCHER(PrinterClass)
{
    switch (msg->MethodID)
    {
        case OM_NEW              : return(mNew   (cl,obj,(APTR)msg));
        case MUIM_Printer_Notify : return(mNotify(cl,obj,(APTR)msg));
    }
}
DISPATCHER_END



/*
** Function which is called to initialize commons just
** after the library was loaded into memory and prior
** to any other API call
**
** This is the good place to open our required libraries
** and create our MUI custom classes
*/
BOOL GUI_InitResources(VOID)
{
    UtilityBase = OpenLibrary("utility.library", 0L);
    IntuitionBase = OpenLibrary("intuition.library", 0L);
    MUIMasterBase = OpenLibrary("muimaster.library", 0L);

    if(UtilityBase && IntuitionBase && MUIMasterBase)
    {
        MCC_PrinterClass = MUI_CreateCustomClass(
            NULL,
            MUIC_Group,
            NULL,
            sizeof(struct PrinterData),
            DISPATCHER_REF(PrinterClass));
    }

    return MCC_PrinterClass != NULL;
}

/*
** Function which is called to free commons just before
** the library is expurged from memory
**
** This is the good place to close our required libraries
** and destroy our MUI custom classes
*/
VOID GUI_FreeResources(VOID)
{
    if(MCC_PrinterClass)
        MUI_DeleteCustomClass(MCC_PrinterClass);

    CloseLibrary(MUIMasterBase);
    CloseLibrary(IntuitionBase);
    CloseLibrary(UtilityBase);
}



/*
** Function which is called from CreatePlugin() to build the GUI
*/
Object * GUI_Create(struct ACEpansionPlugin *plugin)
{
    return PrinterObject,
        MUIA_Printer_Plugin, plugin,
        End;
}

/*
** Function which is called from DeletePlugin() to delete the GUI
*/
VOID GUI_Delete(Object *gui)
{
    MUI_DisposeObject(gui);
}



/*
** Functions called from the plugin
*/
VOID GUI_SetPrinterFileName(Object *gui, CONST_STRPTR fileName)
{
    DoMethod(gui, MUIM_Printer_Notify, MUIV_Printer_Notify_ToGUI,
                  MUIV_Printer_Notify_FileName, fileName);
}

VOID GUI_NotifyPrinterFileError(UNUSED Object *gui, CONST_STRPTR fileName)
{
    MUI_Request(NULL, NULL, 0, GetString(MSG_TITLE), GetString(MSG_OK), GetString(MSG_FILE_ERROR), fileName);
}

